@@ -29,6 +29,9 @@
  *
  * Authors: Federico Mena <federico@nuclecu.unam.mx>
  */
+
+#define _XOPEN_SOURCE 500
+
 #include <config.h>
 #include <glib.h>
 #include <sys/param.h>
@@ -53,7 +56,7 @@
 #include <gtk/gtksignal.h>
 
 /* Must be before all other gnome includes!! */
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
 #include <libgnome/gnome-util.h>
 #include "libgnomeuiP.h"
@@ -457,6 +460,9 @@ browse_dialog_ok (GtkWidget *widget, gpointer data)
 	else
 		locale_filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fw)));
 
+	if (!locale_filename)
+		return;
+
 	utf8_filename = g_filename_to_utf8 (locale_filename, -1, NULL,
 					    NULL, NULL);
 	g_free (locale_filename);
@@ -789,13 +795,10 @@ gnome_file_entry_drag_data_received (GtkWidget        *widget,
 	gtk_entry_set_text (GTK_ENTRY (entry), file);
 }
 
-#define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
-
 static void
 gnome_file_entry_init (GnomeFileEntry *fentry)
 {
 	GtkWidget *button, *the_gtk_entry;
-	static GtkTargetEntry drop_types[] = { { "text/uri-list", 0, 0 } };
 	GtkWidget *hbox;
 
 	gtk_box_set_spacing (GTK_BOX (fentry), 4);
@@ -832,7 +835,9 @@ gnome_file_entry_init (GnomeFileEntry *fentry)
 			   GTK_DEST_DEFAULT_MOTION |
 			   GTK_DEST_DEFAULT_HIGHLIGHT |
 			   GTK_DEST_DEFAULT_DROP,
-			   drop_types, ELEMENTS (drop_types), GDK_ACTION_COPY);
+			   NULL, 0,
+                           GDK_ACTION_COPY);
+        gtk_drag_dest_add_uri_targets (GTK_WIDGET (fentry));
 
 	g_signal_connect (fentry, "drag_data_received",
 			  G_CALLBACK (gnome_file_entry_drag_data_received),
